#!/usr/bin/env python
"""
Head module for Connexion project core
"""

# 	Copyright (c) 2005-2007 ALT Linux, Peter V. Saveliev
#
# 	This file is part of Connexion project.
#
# 	Connexion is free software; you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation; either version 2 of the License, or
# 	(at your option) any later version.
#
# 	Connexion is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with Connexion; if not, write to the Free Software
# 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import os
import sys
## from shell import shell

from bus.dispatch import dispatcher2 as _dispatcher
from bus.monitor import monitor as _monitor
from logger.logger import logger as _logger
from gateway.core import server as _gateway
from gateway.core import proxy, register

from orb.core import orb as _orb
from state.core import manager
from executive.runner import runner as _runner

# commented out due to ALT specific, till it'll be solved
#
# from setproctitle import setproctitle

from getopt import getopt

import time
import threading
import tempfile

# commented out due to ALT specific, till it'll be solved
#
# setproctitle("shell server")

local_shell = False
socketG = "/".join((tempfile.gettempdir(),"socket"))
socketR = "/".join((tempfile.gettempdir(),"register"))
socketT = ""
socketN_a = ""
socketN_p = ""
wdir = []
prefixes = []
flag = None
protodebug = False


def clean():
	try:
		os.unlink(socketG)
		os.unlink(socketR)
		if socketT:
			os.unlink(socketT)
	except:
		pass


def usage():
	print '''
Shell server: the core.

Options:
	-d		turn on server <-> client protocol debug printout
	-h		this help
	-f <path>	flag pipe (see docs before using!)
	-n <addr>	client interface socket addr (tcp/ip, trusted) (see -p)
	-p <port>	client interface port (tcp/ip) (see -n)
	-r <path>	registrator interface socket path
	-s <path>	client interface socket path (untrusted)
	-t <path>	client interface socket path (trusted)
	-w <prefix>:<path>	add modules directory to the tree stack
	-W <prefix>:<path>	add privileged modules directory
	-x		run local debug shell

For details, see the documentation.
'''

try:
	(opts,left) = getopt(sys.argv[1:],"dhf:n:p:r:s:t:w:W:x")
except Exception,e:
	print e
	usage()
	sys.exit(255)

for (i,k) in opts:
	if i == "-d":
		protodebug = True
	elif i == "-h":
		usage()
		sys.exit(0)
	elif i == "-f":
		flag = k
	elif i == "-n":
		socketN_a = k
	elif i == "-p":
		socketN_p = int(k)
	elif i == "-r":
		socketR = k
	elif i == "-s":
		socketG = k
	elif i == "-t":
		socketT = k
	elif i == "-w":
		k = k.split(":")
		k.append(0)
		wdir.append(k)
		prefixes.append(k[0])
	elif i == "-W":
		k = k.split(":")
		k.append(1)
		wdir.append(k)
		prefixes.append(k[0])
	elif i == "-x":
		local_shell = True


## dispatcher thread
dispatcher = _dispatcher("dispatcher2")
dispatcher.start()

## only connect: we do not need to receive any messages
local = dispatcher.connect()

try:
	## logger thread
	logger = _logger("logger")
	logger.bus = dispatcher.connect()
	dispatcher.subscribe(logger.bus.address,"logger")
	logger.start()

	## socket monitor
	monitor = _monitor(dispatcher.connect(),"socket monitor")
	dispatcher.subscribe(monitor.bus.address,"monitor")
	monitor.setDaemon(True)
	monitor.start()

	## background runner
	runner = _runner(dispatcher.connect(),"runner for background jobs")
	dispatcher.subscribe(runner.bus.address,"runner")
	runner.setDaemon(True)
	runner.start()
	
	## orb thread
	orb = _orb(dispatcher.connect(),"orb",wdir)
	dispatcher.subscribe(orb.bus.address,"orb")
	orb.start()

	## state manager:
	state = manager(dispatcher.connect(),"state manager",{"prefixes": prefixes})
	dispatcher.subscribe(state.bus.address,"state")
	state.start()

	## gateway thread
	gateway = _gateway(dispatcher.connect(),proxy,"XML-RPC gateway",socketG,trusted=False,debug=protodebug,prefixes=prefixes)
	gateway.setDaemon(True)
	gateway.start()

	## trusted gateway
	if socketT:
		trusted = _gateway(dispatcher.connect(),proxy,"XML-RPC gateway (trusted)",socketT,trusted=True,debug=protodebug,prefixes=prefixes)
		trusted.setDaemon(True)
		trusted.start()
	
	if socketN_p:
		network = _gateway(dispatcher.connect(),proxy,"XML-RPC gateway (network)",(socketN_a,socketN_p),trusted=True,debug=protodebug,prefixes=prefixes)
		network.setDaemon(True)
		network.start()

	## registrator thread
	registrator = _gateway(dispatcher.connect(),register,"XML-RPC registrator",socketR,debug=protodebug)
	registrator.setDaemon(True)
	registrator.start()

	
##
	if flag:
		flag = open(flag,"w",0)
		flag.close()

##

	## FIXME: the debug shell must be moved out to a separate file/component

	if local_shell:
		print """
Debug shell started. Autocomplete does not work. See 'help' for details
Executable file was started from %s
""" % (sys.argv[0])
		while True:

			try:
				text = raw_input("# ")
			except KeyboardInterrupt:
				print
				text = ""
			except:
				print "quit"
				text = "quit"

			if not text:
				continue

			if text == "threads":
				for i in threading.enumerate():
					print " * %s" % (i.getName())
			elif text == "arp":
				for i in dispatcher.arp():
					print i
			elif text == "groups":
				for i in dispatcher.groups():
					print str(i) + ":"
					for k in dispatcher.subscribers(i):
						print "\t" + str(k)
			elif text == "cache":
				print local.put2("orb",
					{
						"map": "cache"
					}
				)

			elif text == "tree":
				print local.put2("state",
					{
						"map": "trace"
					}
				)

			elif text == "clean":
				print local.put2("orb",
					{
						"map": "clean"
					}
				)

			elif text == "dump":
				print local.put2("dispatcher2","debug")

			elif text == "no dump":
				print local.put2("dispatcher2","no debug")

			elif text == "getacl":
				print local.put2("orb",
					{
						"map": "getacl"
					}
				)

			elif text == "trace":
				print local.put2("runner",
					{
						"map": "debug",
					}
				)

			elif text == "fd":
				os.system("ls -l /proc/%s/fd/" % (os.getpid()))

			elif text == "debug":
				password = raw_input("Enter password for debug connection: ")
				try:
					import rpdb2
					print "starting rpdb2"
					rpdb2.start_embedded_debugger(password)
				except:
					print "no rpdb2 found, continuing w/o debugging"
				
			elif text == "help":
				print '''
Available commands:

	command		(subsystem)	description
	-------		----------	-----------

	arp		(dispatcher)	inspect arp array
	cache		(orb)		inspect object cache
	clean		(orb)		clean object cache
	debug		(main)		run embedded debugger (rpdb2)
	dump		(dispatcher)	turn dispatcher2 packet dumping on
	fd		(main)		inspect open file descriptors
	getacl		(orb)		inspect installed ACLs
	groups		(dispatcher)	inspect multicast groups
	no dump		(dispatcher)	turn dispatcher2 packet dumping off
	threads		(main)		inspect running threads
	trace		(runner)	switch background jobs runner debug
	quit		(main)		stop the server and leave the shell (Ctrl-D)

Embedded debug shell is just very, very simple input interpreter with
no readline/autocomletion support etc.

'''

			elif text == "quit":
				local.put("dispatcher2","shutdown")
				time.sleep(0.5)
				clean()
				sys.exit(0)

			else:
				print "syntax error, see 'help' for details"
	else:
		print '''
This is the debug version of connexion server. You have 5 seconds
to interrupt this launch with Ctrl-C and try '-h'
'''
		time.sleep(5)
		print '''
Now the server can be stopped only with 'shutdown' command from
the connexion console or with explicit kill. Good luck.
'''
		dispatcher.join()

	
except Exception,e:
	#import traceback
	#print e
	#traceback.print_exc()
	local.put("dispatcher2","shutdown")

clean()
